# -*- coding: utf-8 -*-
"""
Created on Mon Apr 12 08:46:06 2021

@author: User
"""
import pandas as pd
import numpy as np
import data_science_helper.helper_output as ho

def drop_cls_unique_value(df):
    ho.print_message("drop_cls_unique_value: Eliminando columnas con un unico valor")
    l=[]
    for col in df.columns:
        if len(df[col].unique()) == 1:
            l.append(col)            
            df.drop(col,inplace=True,axis=1)
    ho.print_items(l,prefix="drop",excepto=[])        
    return df

def drop_corr_columns(X_t,umbral=0.98):
    ho.print_message("drop_corr_columns: Eliminando columnas correlacionales entre si, con umbral {}".format(umbral))
    
    cor_matrix = X_t.corr().abs()
    upper_tri = cor_matrix.where(np.triu(np.ones(cor_matrix.shape),k=1).astype(np.bool))
    to_drop = [column for column in upper_tri.columns if any(upper_tri[column] > umbral)]

    ho.print_items(to_drop,prefix="drop",excepto=[])

    X_t.drop(to_drop, inplace=True, axis=1)
    return X_t



def drop_nan_columns(X_t,p_min_val = .01): #defecto 
    
    temp1 = X_t.columns    
    thresh = len(X_t) * p_min_val
    ho.print_message("drop_nan_columns: elimina columnas con menos de {}% ({}) registros validos".format(p_min_val*100, thresh))
    X_t.dropna(thresh = thresh, axis = 1, inplace = True)
    temp2 = X_t.columns     
    l = list(set(temp1) - set(temp2))
    
    ho.print_items(l,prefix="drop",excepto=[])


    return X_t



import matplotlib.pylab as plt
import matplotlib.cm as cm

from sklearn import preprocessing
import seaborn as sns
import pandas as pd
import pyodbc
import numpy as np
import pyodbc
import missingno as msno
from sklearn.decomposition import PCA
from sklearn.metrics import silhouette_score,silhouette_samples
from sklearn.manifold import TSNE
from sklearn.cluster import KMeans
from sklearn.cluster import DBSCAN
from sklearn.cluster import SpectralClustering
from scipy.stats import itemfreq
from sklearn.base import BaseEstimator, TransformerMixin
import pandas as pd



class ClusterTransform( BaseEstimator, TransformerMixin ):
    #Class Constructor                                                               
    def __init__( self,  cluster_model='KMeans',embedded="TSNE",progreso_score=False,perplexity=40,
                 n_clusters=20,verbose=1,start=1 ,limit=50,step=5,label_group=None,color_map=None ):
        
        #self._feature_names = feature_names 
        self._progreso_score = progreso_score
        self._n_clusters = n_clusters
 
        self._model = None
        self._verbose = verbose
        self._start = start
        self._limit = limit
        self._step = step
        self._perplexity = perplexity
        self._cluster_model = cluster_model
        self._embedded = embedded
        
        self._label_group = label_group
        self._color_map = color_map
    
    #Return self nothing else to do here    
    def fit( self, X, y = None ):
                     

        
        return self 
    
    #Method that describes what we need this transformer to do
    def transform( self, X, y = None ):
        
        print("inicio del proceso: cluster_summary ")     
        
        labels = cluster_summary(X,embedded = self._embedded,
                                      perplexity = self._perplexity,
                                      cluster_model = self._cluster_model,                                      
                                      progreso_score = self._progreso_score,
                                      n_clusters = self._n_clusters,
                                      start=self._start ,limit=self._limit,step=self._step,
                                      label_group = self._label_group,
                                      color_map = self._color_map,
                                )
        
        return labels

            
    def importances_df(self):
        return self._importance_df




def cluster_summary(df=None, embedded='TSNE',perplexity=40,cluster_model='KMeans',n_clusters=30
                 ,progreso_score=False,start=1 ,limit=50,step=5,embedded_in_cluster=True,plot=True,
                    return_silhouette_score=False,label_group=None,color_map=None):
    
        random_state=42
        df_text = df.copy()

        X = df_text.copy()
        
        data_vector = X.values

        if(embedded=='TSNE'):
            X_embedded = TSNE(n_components=2, perplexity=perplexity, verbose=0).fit_transform(data_vector)             
            #try data_vector or X_reduced
            #verbose->2 para mostrar log
        else:
            pca = PCA(n_components=2, random_state=random_state)
            X_embedded = pca.fit_transform(data_vector)
            
        if embedded_in_cluster:
            data_vector = X_embedded

        print(cluster_model)
        if(cluster_model=='KMeans'):        
            clustering_model = KMeans(n_clusters=n_clusters, max_iter=200, n_init=100,random_state=42)
            labels = clustering_model.fit_predict(data_vector)
            
        elif(cluster_model=='DBSCAN'):             
            clustering_model = DBSCAN().fit(data_vector)
            labels = clustering_model.labels_
        elif(cluster_model=='SpectralClustering'): 
            clustering_model=SpectralClustering(n_clusters= n_clusters, affinity='nearest_neighbors')
            labels = clustering_model.fit_predict(data_vector)
        
        if label_group==None:
            labels_names = ["Class_" + str(x) for x in labels]
        else:
            labels_names = list(map(label_group.get, labels))
        
        if plot:
            colors = plot_cluster(X=X_embedded,range_n_clusters=[n_clusters],
                                  clusterer=clustering_model,cluster_labels=labels,color_map=color_map)
            fig = plt.figure(figsize=(8, 5))
            
            if color_map==None:
                colors = colors.tolist()
            print("-----------------")
            #print(len(labels))
            #print(len(colors)) 
            print("-----------------")
            
            #print(labels)
            #print(colors.head())
            DATA = {'label': labels_names,
                    'color': colors
                   }
            df_l = pd.DataFrame(DATA, columns = ['label','color'])
            df_l['total'] = 1
            if color_map==None:
                df_l['color'] = [tuple(x) for x in df_l.color.values]
          
            df_tem_g = df_l.groupby(['label','color']).sum().reset_index().sort_values(by=['total'], ascending=True)
            df_tem_g['perc']= df_tem_g['total']/df_tem_g['total'].sum()                      
            
            fig = plt.figure(figsize=(7, 5))
            
            x = df_tem_g.label.tolist()
            y = df_tem_g.total.tolist()         
            colors_g = df_tem_g.color.tolist()
            print(colors_g)
            plt.barh(x, y,color=colors_g)

            plt.xlabel("Total", fontsize="x-large")
            plt.ylabel("Grupos", fontsize="x-large")

            for index, value in enumerate(y):
                plt.text(value, index, "  "+str(value)+" ", fontsize=15)
            
            #plt.hist(labels, bins=n_clusters)
            plt.show() 
            
            fig = plt.figure(figsize=(8, 6))
            fig1, ax1 = plt.subplots()
            fig1 = plt.figure(figsize=(8, 6))
            wedges, texts, autotexts =  ax1.pie(df_tem_g['perc'].tolist(),labels=x, autopct='%1.1f%%',
                    shadow=True, startangle=90,colors=colors_g,textprops=dict(color="w"))
            plt.setp(autotexts, size=12, weight="bold")
            ax1.axis('equal')  # Equal aspect ratio ensures that pie is drawn as a circle.

            plt.show()

            
        
            print("Frecuencia : ")
            print("*********************************************************************************")
            cluster , frecuency = np.unique(labels, return_counts=True)
            cls_fre = zip(cluster,frecuency)
            for i in cls_fre:    
                print("[",i[0],"-",i[1],"]", end = ", ") 
            
        #return df_l

        if(progreso_score):
            #buscando numero de cluster ideal
            X_=data_vector
            scores = []
            scores_silueta = []           
            for k in range (start, limit,step):
                if(cluster_model=='KMeans'): 
                    kmeans = KMeans(n_clusters= k, max_iter=200, n_init=100).fit(X_)
                    score = kmeans.score(X_)
                    scores.append((k,score))
                    ssc= silhouette_score(X_, labels=kmeans.predict(X_))
                    scores_silueta.append((k,ssc)) 
                if(cluster_model=='SpectralClustering'): 
                    labels_=SpectralClustering(n_clusters= k, affinity='nearest_neighbors').fit_predict(data_vector)
                    ssc= silhouette_score(X_, labels=labels_)
                    scores_silueta.append((k,ssc))
                
            scores = np.array(scores)
            scores_silueta = np.array(scores_silueta)

            fig = plt.figure(figsize=(18, 5))
            ax1 = fig.add_subplot(121)
            plt.title('Progreso de scores_silueta')
            plt.plot(scores_silueta[:,0], scores_silueta[:,1],  marker='o')
            
            if(cluster_model=='KMeans'):
                ax2 = fig.add_subplot(122)
                plt.title('Elbow Curve')
                plt.plot(scores[:,0], scores[:,1],  marker='o') 

            plt.show()           

        ssc= silhouette_score(data_vector, labels=labels)    
        print("silhouette_score : "+str(ssc))
            
        if return_silhouette_score:
            return ssc
        else:       
            return labels_names

    

    
    
    
def plot_cluster(X=None,range_n_clusters=[],clusterer=None,cluster_labels=[],color_map=None):
    colors = []
    for n_clusters in range_n_clusters:
        # Create a subplot with 1 row and 2 columns
        fig, (ax1, ax2) = plt.subplots(1, 2)
        fig.set_size_inches(18, 7)

        # The 1st subplot is the silhouette plot
        # The silhouette coefficient can range from -1, 1 but in this example all
        # lie within [-0.1, 1]
        ax1.set_xlim([-0.1, 1])
        # The (n_clusters+1)*10 is for inserting blank space between silhouette
        # plots of individual clusters, to demarcate them clearly.
        ax1.set_ylim([0, len(X) + (n_clusters + 1) * 10])

        # Initialize the clusterer with n_clusters value and a random generator
        # seed of 10 for reproducibility.
        ###clusterer = KMeans(n_clusters=n_clusters, random_state=10)
        ###cluster_labels = clusterer.fit_predict(X)

        # The silhouette_score gives the average value for all the samples.
        # This gives a perspective into the density and separation of the formed
        # clusters
        silhouette_avg = silhouette_score(X, cluster_labels)
        print("For n_clusters =", n_clusters,
              "The average silhouette_score is :", silhouette_avg)

        # Compute the silhouette scores for each sample
        sample_silhouette_values = silhouette_samples(X, cluster_labels)

        y_lower = 10
        for i in range(n_clusters):
            # Aggregate the silhouette scores for samples belonging to
            # cluster i, and sort them
            ith_cluster_silhouette_values = \
                sample_silhouette_values[cluster_labels == i]

            ith_cluster_silhouette_values.sort()

            size_cluster_i = ith_cluster_silhouette_values.shape[0]
            y_upper = y_lower + size_cluster_i
            
            if color_map == None:
                color = cm.nipy_spectral(float(i) / n_clusters)
            else:
                color = color_map[i]
                
            ax1.fill_betweenx(np.arange(y_lower, y_upper),
                              0, ith_cluster_silhouette_values,
                              facecolor=color, edgecolor=color, alpha=0.7)

            # Label the silhouette plots with their cluster numbers at the middle
            ax1.text(-0.05, y_lower + 0.5 * size_cluster_i, str(i))

            # Compute the new y_lower for next plot
            y_lower = y_upper + 10  # 10 for the 0 samples

        ax1.set_title("The silhouette plot for the various clusters.")
        ax1.set_xlabel("The silhouette coefficient values")
        ax1.set_ylabel("Cluster label")

        # The vertical line for average silhouette score of all the values
        ax1.axvline(x=silhouette_avg, color="red", linestyle="--")

        ax1.set_yticks([])  # Clear the yaxis labels / ticks
        ax1.set_xticks([-0.1, 0, 0.2, 0.4, 0.6, 0.8, 1])

        if color_map == None:
            # 2nd Plot showing the actual clusters formed
            colors = cm.nipy_spectral(cluster_labels.astype(float) / n_clusters)
        else:
            colors = list(map(color_map.get, cluster_labels))
        
        ax2.scatter(X[:, 0], X[:, 1], marker='.', s=30, lw=0, alpha=0.7,
                    c=colors, edgecolor='k')

        # Labeling the clusters
        centers = clusterer.cluster_centers_
        #print(centers)
        # Draw white circles at cluster centers
        ax2.scatter(centers[:, 0], centers[:, 1], marker='o', c="white", alpha=1, s=200, edgecolor='k')

        for i, c in enumerate(centers):
            ax2.scatter(c[0], c[1], marker='$%d$' % i, alpha=1,
                        s=50, edgecolor='k')

        ax2.set_title("The visualization of the clustered data.")
        ax2.set_xlabel("Feature space for the 1st feature")
        ax2.set_ylabel("Feature space for the 2nd feature")

        plt.suptitle(("Silhouette analysis for KMeans clustering on sample data "
                      "with n_clusters = %d" % n_clusters),
                     fontsize=14, fontweight='bold')

    plt.show()
    return colors



import shap
import matplotlib.pylab as plt
from sklearn.preprocessing import StandardScaler, LabelEncoder , LabelBinarizer
import seaborn as sns
import pandas as pd
import pyodbc
import numpy as np
import pyodbc
import missingno as msno
from sklearn.decomposition import PCA
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.decomposition import TruncatedSVD

from data_science_helper import helper_dataframe as hd

import string
from sklearn.metrics import silhouette_score
from sklearn.manifold import TSNE
from sklearn.cluster import KMeans
from sklearn.cluster import DBSCAN
from sklearn.cluster import SpectralClustering
from scipy.stats import itemfreq
from sklearn.base import BaseEstimator, TransformerMixin
from sklearn.model_selection import train_test_split
import lightgbm as lgb
import pandas as pd

from sklearn import preprocessing, decomposition, svm #, cross_validation


class ShapFeatureSelector( BaseEstimator, TransformerMixin ):
    #Class Constructor 
    def __init__( self,  objective="binary",min_shap=0,test_size=0.25,verbose=1,categorical_columns_name=[] ):
        #self._feature_names = feature_names 
        self._objective = objective
        self._min_shap = min_shap
        self._test_size=test_size
        self._model = None
        self._verbose = verbose
        self._categorical_columns_name = categorical_columns_name
    
    #Return self nothing else to do here    
    def fit( self, X, y = None ):
        
        print("inicio del proceso: feature_sel_shap con  min_shap:"+str(self._min_shap))
        train_cols = X.columns.tolist()

#         _cols = X.columns
#         _num_cols = X._get_numeric_data().columns
#         _categorical_columns_name = list(set(_cols) - set(_num_cols))
        
#         le = preprocessing.LabelEncoder()
#         for c in _categorical_columns_name:
#             col_type = X[c].dtype
#             if col_type == 'object' or col_type.name == 'category':
#                 #print("column : "+str(c))
#                 X[c] = le.fit_transform(X[c])
#                 #X[c] = X[c].astype('category') 
        
        
        X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=self._test_size, stratify=y, random_state=1)

        
        train_data = lgb.Dataset(X_train, label=y_train.astype(int), feature_name=train_cols)
        test_data = lgb.Dataset(X_test, label=y_test.astype(int), feature_name=train_cols, reference=train_data)
        
        # LGB parameters:
        params = {
                'metric': 'binary_error',
                'learning_rate': 0.01,
                'boosting': 'gbdt', 
                'objective': self._objective,
                'num_leaves': 2000,
                'min_data_in_leaf': 200,
                'max_bin': 200,
                'max_depth': -1,
                'seed': 2018,
                'nthread': 10,
                "lambda_l1": 0.1,            
                }
        
        self._model = lgb.train(params, train_data, 
                    categorical_feature=self._categorical_columns_name,
                    num_boost_round=1000, 
                    valid_sets=(test_data,), 
                    valid_names=('valid',), 
                    verbose_eval=False, 
                    early_stopping_rounds=20)
        
        return self 
    
    #Method that describes what we need this transformer to do
    def transform( self, X, y = None ):
        
        if(self._model is not None):
            explainer = shap.TreeExplainer(self._model)
            shap_values = explainer.shap_values(X)

            if(self._objective=="multiclass"):
                shap_sum = np.abs(shap_values).mean(axis=0).mean(axis=0)
            else:  
                shap_sum = np.abs(shap_values).mean(axis=0).mean(axis=0)

            #print("shap_sum...done")
            importance_df = pd.DataFrame([X.columns.tolist(), shap_sum.tolist()]).T
            importance_df.columns = ['column_name', 'shap_importance']
            importance_df = importance_df.sort_values('shap_importance', ascending=False)

            importance_df['percentage'] = (importance_df['shap_importance']/importance_df['shap_importance'].sum())*100
            importance_df.reset_index(inplace=True,drop=True)
            if(self._verbose==1):
                print(importance_df)
            
            
            zero_features = importance_df[importance_df['shap_importance']<=self._min_shap].column_name.values
            return X.drop(columns = zero_features)     
        else:
            print("No se ha ejecutado el metodo FIT")
            
    def importances_df(self):
        return self._importance_df


class CorrTransformer( BaseEstimator, TransformerMixin ):
    #Class Constructor 
    def __init__( self, umbral  = 0.95 ):
        self.to_drop  = []
        self.umbral = umbral
    
    #Return self nothing else to do here    
    def fit( self, X, y = None ):   
        
        c_c = hd.get_cat_columns(X)
        
        # Create correlation matrix
        corr_matrix = X.drop(c_c,axis=1).corr().abs()
        # Select upper triangle of correlation matrix
        upper = corr_matrix.where(np.triu(np.ones(corr_matrix.shape), k=1).astype(np.bool))
        # Find index of feature columns with correlation greater than 0.95
        to_drop = [column for column in upper.columns if any(upper[column] > self.umbral)]
        self.to_drop = to_drop
        return self 
    
    #Method that describes what we need this transformer to do
    def transform( self, X, y = None ):
        
        new_df = X.drop(self.to_drop,axis=1,inplace=False)  
        return new_df
        
     
       
    
class CatTransformer( BaseEstimator, TransformerMixin ):
    #Class Constructor 
    def __init__( self, pp  = "le" , cl_list=None, console= False ):
        
        #columns_cat
        self.cat_encoders  = []
        self.encoded_df = []
        self.pp = pp
        self.console = console
        self.cl_list = cl_list
    
    #Return self nothing else to do here    
    def fit( self, X, y = None ):
        if self.cl_list is None:
            self.catcolumns = hd.get_cat_columns(X)
        else:
            self.catcolumns = self.cl_list 

        for name in self.catcolumns:
            
            if name in X.columns:
            
                if(self.pp=="le"):
                    le = LabelEncoder()  
                elif(self.pp=="lb"):
                    le = LabelBinarizer()  
                else:
                    msg = "Error: pp INVALIDO"
                    raise Exception(msg)
                    break;
                if self.console==True:
                    print("NEW Binarios {}".format(name))
                if X[name].isna().any():
                    msg = "ERROR: La columna "+name+" tiene valores MISSING, corregir eso para continuar"
                    raise Exception(msg)
                    #print("ERROR: La columna ",name," tiene valores MISSING, corregir eso para continuar")
                    break
                le.fit(X[name])
                self.cat_encoders.append((name, le))
            else:
                print(f"no se transformara la columna {name} porque no existe en el dataframe" )
        return self 
    
    #Method that describes what we need this transformer to do
    def transform( self, X, y = None ):
        
        #print(self.cat_encoders)
        for name, encs in self.cat_encoders :
            
            df_c = encs.transform(X[name])
            
            prefix = str(name) + "_"

            if(self.pp=="le"):
                self.encoded_df.append(pd.DataFrame(df_c,columns=[name],index=X.index))
            elif(self.pp=="lb"):
                #self.encoded_df.append(pd.DataFrame(df_c,columns=encs.classes_,index=X.index).add_prefix(str(prefix).upper()))
                if(df_c.shape[1]==1):                    
                    if (len(encs.classes_)==2):  
                        classe_1 = encs.classes_[1]+"_pos_"+encs.classes_[0]+"_neg"
                        self.encoded_df.append(pd.DataFrame(df_c,columns=[classe_1],index=X.index).add_prefix(str(prefix).upper()))
                    else:
                        self.encoded_df.append(pd.DataFrame(df_c,columns=[name],index=X.index))                    
                else:                
                    self.encoded_df.append(pd.DataFrame(df_c,columns=encs.classes_,index=X.index).add_prefix(str(prefix).upper()))          
            
            else:
                print("Error: pp INVALIDO")
                break;            
            
        encoded_df_c = pd.concat(self.encoded_df, axis = 1, ignore_index  = False)   
        #print(list(encoded_df_c.columns))
        #print(self.encoded_df.columns)
       
        self.df_num = X.drop(self.catcolumns, axis = 1) 
        #print(self.df_num.shape)
        #print(list(self.df_num.columns))
        new_df = pd.concat([self.df_num, encoded_df_c], axis = 1, ignore_index = False)
        
        #para que las columnas categoricas codificadas con numeros no se confundan con valores numericos, le asignamos tipo category
        if(self.pp=="le"):
            for c in self.catcolumns:
                new_df[c] = new_df[c].astype('category') 
        
        #print(new_df.shape)
        #print(list(y.columns))
        return new_df
        
     
        
        
        
class FeatureSelector( BaseEstimator, TransformerMixin ):
    #Class Constructor 
    def __init__( self, feature_names ):
        self._feature_names = feature_names 
    
    #Return self nothing else to do here    
    def fit( self, X, y = None ):
        return self 
    
    #Method that describes what we need this transformer to do
    def transform( self, X, y = None ):
        return X[ self._feature_names ] 


# -*- coding: utf-8 -*-

from sklearn.metrics import confusion_matrix, roc_auc_score, classification_report,average_precision_score , f1_score
from sklearn.metrics import roc_curve, precision_recall_curve, auc,recall_score,precision_score
import pickle
from sklearn.model_selection import StratifiedKFold
import seaborn as sns
import numpy as np
import pandas as pd
import os
import json
import data_science_helper.helper_general as hg
import lightgbm as lgb 
#hg.set_base_path()

'''   
MR_File_Name = "base.txt"

def set_macro_region(filename):
    global MR_File_Name
    MR_File_Name = filename

def get_macro_region(macro):
    global MR_File_Name
    js_mr = hg.get_base_path()+"/config/macro_region/"+MR_File_Name
    with open(js_mr) as json_file:
        data = json.load(json_file)        
        
    if macro not in data: 
            raise Exception("No existe la clave " + macro + " en el archivo: "+MR_File_Name)
        
    return data[macro]
    
 
    if(macro=="lima"):
        return ['DRE LIMA METROPOLITANA','DRE LIMA PROVINCIAS','DRE CALLAO']
    elif(macro=="norte"):
        return ['DRE ANCASH','DRE CAJAMARCA','DRE LA LIBERTAD','DRE LAMBAYEQUE','DRE PIURA','DRE TUMBES']
    elif(macro=="sur"):
        return ['DRE AREQUIPA','DRE CUSCO','DRE MADRE DE DIOS','DRE MOQUEGUA','DRE PUNO','DRE TACNA']   
    elif(macro=="centro"):
        return ["DRE APURIMAC","DRE AYACUCHO","DRE HUANCAVELICA",'DRE HUANUCO','DRE JUNIN','DRE PASCO','DRE ICA']
    elif(macro=="oriente"):
        return ['DRE AMAZONAS','DRE LORETO','DRE SAN MARTIN','DRE UCAYALI']
'''


def get_opt_threshold(y_test=None,y_pred_prob=None,intervalo = 0.001 ):
    thresholds = np.arange(0.0, 1.0, intervalo)
    score = np.zeros(shape=(len(thresholds)))
    print('Length of sequence: {}'.format(len(thresholds)))

    # Fit the model
    for index, elem in enumerate(thresholds):
        # Corrected probabilities
        y_pred = (y_pred_prob > elem).astype('int')
        #print(y_pred_prob)
        # Calculate the f-score
        score[index] = f1_score(y_test, y_pred)
        #score[index] = average_precision_score(y_test, y_pred_prob)
        #break


    # Find the optimal threshold
    index = np.argmax(score)
    print(thresholds[index])
    thresholdOpt = round(thresholds[index], ndigits = 4)
    scoreOpt = round(score[index], ndigits = 4)
    print('Best Threshold: {} with F-Score: {}'.format(thresholdOpt, scoreOpt))
    return thresholdOpt



def f1_eval_metric(y_true, y_pred):

    eval_result = f1_score(y_true, y_pred.round())
    return ("f1", eval_result, True)

def precision_eval_metric(y_true, y_pred):

    eval_result = precision_score(y_true, y_pred.round())
    return ("precision", eval_result, True)      

def recall_eval_metric(y_true, y_pred):

    eval_result = recall_score(y_true, y_pred.round())
    return ("recall", eval_result, True)

def average_precision_eval_metric(y_true, y_pred):

    eval_result = average_precision_score(y_true, y_pred)
    return ("average_precision", eval_result, True)

def roc_auc_eval_metric(y_true, y_pred):

    eval_result = roc_auc_score(y_true, y_pred)
    return ("roc_auc", eval_result, True)

def get_fn_eval(score_rs):     
    if(score_rs=="f1"):
        fn_eval=f1_eval_metric
    elif(score_rs=="precision"):
        fn_eval=precision_eval_metric
    elif(score_rs=="recall"):
        fn_eval=recall_eval_metric
    elif(score_rs=="average_precision"):
        fn_eval=average_precision_eval_metric  
    elif(score_rs=="roc_auc"):
        fn_eval=roc_auc_eval_metric
    #else:
    #    fn_eval=custom_eval_metric
        
    return fn_eval


def get_hypers_dic(url,common_params):  
    hypers_dic = {}
    dir_list = os.listdir(url)    
    for file_name in dir_list:
        name = os.path.splitext(file_name)[0]    
        file_path  = url+file_name
        #print(file_path)
        with open(file_path) as json_file:
            data = json.load(json_file)    
            for key_mr , value_mr in data.items():
                for key_mr_model, value_mr_model in value_mr.items():
                    data[key_mr][key_mr_model] = {**common_params, **value_mr_model }
                    #data[key_mr][model_name] = {**common_params, **data[key_mr][model_name] }
           
            hypers_dic[name] = data  
            
    return hypers_dic


def get_hypers_gr_mr_dic(hyperparameters_dir,key,macro_region,model_name,params):
    if hyperparameters_dir is None:
        print("\nNo se especifico ruta para hyperparametros , se empleara valores por defecto\n")
        return params
    hypers_dic = get_hypers_dic(hyperparameters_dir, params)
    
    hypers_dic_gr = hypers_dic.get(key, None)
    if hypers_dic_gr is None:
        print("\nNo existe hyperparametros para {",key, "}, se empleara valores por defecto\n")
        return params
    else:
        hypers_dic_gr_mr = hypers_dic_gr.get(macro_region, None)
        if hypers_dic_gr_mr is None:
            print("\nNo existe hyperparametros para {",key,"-",macro_region, "}, se empleara valores por defecto\n")
            return params
        else:
            hypers_dic_gr_mr_model = hypers_dic_gr_mr.get(model_name, None)
            if hypers_dic_gr_mr_model is None:
                print("\nNo existe hyperparametros para {",key,"-",macro_region, "-",model_name,"}, se empleara valores por defecto\n")
                return params
            else:
                return hypers_dic_gr_mr_model
        
def save_hyper_draft(best_params_full,kpis_dict_full,hyper_draft_dir,key):
    save_json(best_params_full,hyper_draft_dir+key+".json") 
    save_json(kpis_dict_full,hyper_draft_dir+key+"_kpi.json") 
 

def save_json(dict_obj, url_file):
    if url_file is None:
        print("")
    else:
        # create json object from dictionary
        json_ = json.dumps(dict_obj, sort_keys=True, indent=4)
        # open file for writing, "w" 
        f = open(url_file,"w")
        # write json object to file
        f.write(json_)
        # close file
        f.close() 

def get_kfold():
    n_splits = 4
    kfold = StratifiedKFold(n_splits=n_splits, shuffle=True,random_state=2)
    return kfold

def save_model(model,dir_name):    
    if dir_name is None:
        print("")
    else:
        filename ='model.{}' 
        
        if len(dir_name.strip())==0 :
            full_dirname = filename
        else:
            if os.path.isdir(dir_name)==False:
                os.makedirs(dir_name)
            full_dirname = os.path.join(dir_name, filename)        
        
        if type(model)==lgb.basic.Booster:            
            model.save_model(full_dirname.format("booster"))            
        else:                  
            pickle.dump(model, open(full_dirname.format("sav"), 'wb'))   
    
def load_save_model(url_dir):
    filename = url_dir+'/model.sav'
    loaded_model = pickle.load(open(filename, 'rb'))
    return loaded_model





def generate_summary_evaluation(X,y_prob_uno,y,dir_name,modalidad="EBR"):
    
    if dir_name is None:
        return 
    
    #y_prob_uno = predicted_probas[:,1]
    y_pred = np.round(y_prob_uno, 0)
    

    X_eval = X.copy()
    #X_eval['PREDICT_PROBA'] = estimator_random.predict_proba(X)[:,1]
    #X_eval['PREDICT'] = estimator_random.predict(X)
    
    X_eval['PREDICT_PROBA'] = y_prob_uno
    X_eval['PREDICT'] = list(y_pred)
    
    

    X_eval['REAL'] =  y

    criteria = [X_eval['PREDICT_PROBA'].between(0,0.1),
                X_eval['PREDICT_PROBA'].between(0.1, 0.2), 
                X_eval['PREDICT_PROBA'].between(0.2, 0.3),
                X_eval['PREDICT_PROBA'].between(0.3, 0.4),
                X_eval['PREDICT_PROBA'].between(0.4, 0.5),
                X_eval['PREDICT_PROBA'].between(0.5, 0.6),
                X_eval['PREDICT_PROBA'].between(0.6, 0.7),
                X_eval['PREDICT_PROBA'].between(0.7, 0.8),
                X_eval['PREDICT_PROBA'].between(0.8, 0.9),
                X_eval['PREDICT_PROBA'].between(0.9,0.95),
                X_eval['PREDICT_PROBA'].between(0.95, 1)
               ]

    values = ['0% - 10%', '10% - 20%', '20% - 30%','30% - 40%',
              '40% - 50%','50% - 60%','60% - 70%','70% - 80%','80% - 90%','90% - 95%','95% - 100%']

    values_index = [1, 2,3,4,5,6,7,8,9,10,11]
    
    ###################################################################
    
    criteria2 = [X_eval['PREDICT_PROBA'].between(0.0, 0.5),
                X_eval['PREDICT_PROBA'].between(0.5, 1),             
               ]

    values2 = ['0% - 50%', '50% - 100%']

    values_index2 = [1, 2]
    
    #####################################################
 
    X_eval['PREDICT_CATEGORY'] = np.select(criteria, values, 0)
    X_eval['PREDICT_CATEGORY_INDEX'] = np.select(criteria, values_index, 0)

    X_eval['PREDICT_CATEGORY2'] = np.select(criteria2, values2, 0)
    X_eval['PREDICT_CATEGORY_INDEX2'] = np.select(criteria2, values_index2, 0)

    X_eval['TOTAL'] = 1     

    
    X_eval['ACIERTO(VP)'] = np.where( (X_eval['REAL']==1) & (X_eval['PREDICT']==1), 1 ,0)
    X_eval['FP'] = np.where( (X_eval['REAL']==0) & (X_eval['PREDICT']==1), 1 ,0)    

    X_eval_gb = X_eval.groupby(['PREDICT_CATEGORY_INDEX','PREDICT_CATEGORY']).agg({'TOTAL':'count','REAL':'sum'}).reset_index()
    X_eval_gb.sort_values(by='PREDICT_CATEGORY_INDEX', ascending=False,inplace=True)
    X_eval_gb.rename(columns={'REAL': 'DESERTORES TEST','TOTAL': 'TOTAL TEST','PREDICT_CATEGORY':'RANGO'}, inplace=True)
    del X_eval_gb['PREDICT_CATEGORY_INDEX']
    X_eval_gb.reset_index(drop=True,inplace=True)

    X_eval_gb['TOTAL TEST %']=round(X_eval_gb['TOTAL TEST']/X_eval_gb['TOTAL TEST'].sum(),3)
    X_eval_gb['COBERTURA %']=round(X_eval_gb['DESERTORES TEST']/X_eval_gb['DESERTORES TEST'].sum(),3)


    X_eval_gb['PRECISION RANGO %']=round(X_eval_gb['DESERTORES TEST']/X_eval_gb['TOTAL TEST'],3)

    
    X_eval_gb['TOTAL TEST %'] = X_eval_gb['TOTAL TEST %']*100 
    X_eval_gb['PRECISION RANGO %'] = X_eval_gb['PRECISION RANGO %']*100 
    X_eval_gb['COBERTURA %'] = X_eval_gb['COBERTURA %']*100 
    
    #X_eval_gb.index.name = 'N'


    # Set colormap equal to seaborns light green color palette


    if dir_name is None:
        cm = sns.light_palette("red", as_cmap=True)
        return (X_eval_gb.style
                        .background_gradient(cmap=cm, subset=['PRECISION RANGO %'])
                        #.format({'TOTAL TEST %': "{:.1%}"})  
                        #.format({'PRECISION RANGO %': "{:.1%}"})
                        #.format({'COBERTURA %': "{:.1%}"})
                       )
    else:
        
        filename ='df_agg_t_{}.xlsx'
        if len(dir_name.strip())==0 :
            full_dirname = filename
        else:
            if os.path.isdir(dir_name)==False:
                os.makedirs(dir_name)
            full_dirname = os.path.join(dir_name, filename)  
        
        X_eval_gb.to_excel(full_dirname.format(modalidad), encoding="utf-8") 
        
        if(modalidad=="EBR"):
         #   X_eval_gb.to_excel(url_dir+"/df_agg_t.xlsx", encoding="utf-8")    
            get_min_prob_df(y_prob_uno,y,dir_name)
        #else:
        #    X_eval_gb.to_excel(url_dir+"/df_agg_t_{}.xlsx".format(modalidad), encoding="utf-8")  
            

           
def get_min_prob_df(y_prob_uno,y_test,dir_name):
    
    if dir_name is None:
        return

    #y_prob_uno = predicted_probas[:,1]
    y_pred = np.round(y_prob_uno, 0)

    df = pd.DataFrame()
    df['y_prob_uno']  = y_prob_uno
    df['y_pred']  = y_pred
    df['y_test']  = list(y_test)  
    df.sort_values(by=['y_prob_uno'],ascending=False,inplace=True)
    #print(df.head(20))
    total_deser = np.sum(y_test)  


    list_y_true = []
    list_y_pred = []
    list_precision = []
    prob_min = 0
    t_test_min = 0
    prec = 0 
    for index, row in df.iterrows(): 
        list_y_true.append(row['y_test'])
        list_y_pred.append(row['y_pred'])
        pr = precision_score(list_y_true, list_y_pred, average='binary')
        list_precision.append(pr)
        if(pr<1):
            break
        else:
            prob_min = row['y_prob_uno']
            t_test_min = len(list_y_true)
            prec = pr


    df_min = pd.DataFrame()
    df_min['PROB MIN']  = [prob_min]
    df_min['TOTAL DESERTORES TEST RANGO']  = [t_test_min]
    df_min['TOTAL DESERTORES TEST']  = [int(total_deser)]
    df_min['PRECISION RANGO %']  = [int(prec*100)]
    #return df_min
    
    filename ='df_agg_t_min.xlsx'
    if len(dir_name.strip())==0 :
        full_dirname = filename
    else:
        if os.path.isdir(dir_name)==False:
            os.makedirs(dir_name)
        full_dirname = os.path.join(dir_name, filename) 
    
    df_min.to_excel(full_dirname, encoding="utf-8")


# -*- coding: utf-8 -*-
"""
Created on Tue Mar 23 15:44:39 2021

@author: User
"""
#import core_helper.helper_general as hg
#hg.set_base_path()

from data_science_helper import helper_general as hg

import statistics
import json
import os
import math
import sys
import math
import os
import pandas as pd
import numpy as np
import time
from datetime import datetime
import matplotlib.pyplot as plt
import seaborn as sns
import lightgbm as lgb 
#import core_helper.helper_plot as hp
#import src.Prj_Core.core_helper.helper_plot as hp
from data_science_helper import helper_plot as hp
import data_science_helper.helper_dataframe as hd

#import model.general as g
#import src.Prj_Core.core_helper.model.general as g
from data_science_helper.model import general as g
from data_science_helper.model import neg_bagging_fraction__lgb_model as nbf_lgb_model
from data_science_helper.model import scale_pos_weight__lgb_model as spw_lgb_model
from data_science_helper.model import custom_bagging__lgb_model as cb_lgb_model
from sklearn.model_selection import StratifiedKFold   
from sklearn.metrics import f1_score

from data_science_helper.model import lgb_model as l

from sklearn.metrics import precision_recall_fscore_support
from sklearn.metrics import accuracy_score, auc, average_precision_score, confusion_matrix, roc_curve, precision_recall_curve
from numpy import interp
from sklearn.metrics import  roc_auc_score, average_precision_score 
from sklearn.model_selection import  StratifiedShuffleSplit
import seaborn as sns
import shutil
#import src.Prj_Core.core_helper.model.neg_bagging_fraction__lgb_model as nbf_lgb_model
#import src.Prj_Core.core_helper.model.scale_pos_weight__lgb_model as spw_lgb_model
#import src.Prj_Core.core_helper.model.custom_bagging__lgb_model as cb_lgb_model

#import model.neg_bagging_fraction__lgb_model as nbf_lgb_model
#import model.scale_pos_weight__lgb_model as spw_lgb_model
#import model.custom_bagging__lgb_model as cb_lgb_model

def modelar_clasificacion_binaria(strategy="", X_train=None,y_train=None,X_test=None,y_test=None,params=None,max_iter_cls=None,
                                  cls_ini_opt=[] ,enable_cls_opt = False, enable_umbral_opt=False,metric='average_precision',
                                  num_rounds=10000,early_stop=400,log=0,api="train_api",sesgo=0.15,random_state=42,url=None, 
                                  cv=False,cv_n_split=None, print_consola=True,   shuffle_cv=False,  shuffle_cv_test_size=None):
    start = time.time()
    
    if url is not None:        
        #shutil.rmtree(url)
        hg.validar_directorio(url)      
    
    params.update({"metric":metric})
    params.update({"seed":random_state})
    params.update({"objective":'binary'})
            
    args = {'X_train':X_train, 'y_train': y_train, 'X_test': X_test, 'y_test': y_test, 'params':params,
            "metric":metric,"num_rounds":num_rounds,"early_stop":early_stop,"log":log,"random_state":random_state,"api":api ,"strategy":strategy,
            "url":url }     
    
    list_cls_ini = None
    
    if (enable_cls_opt):
        print("----------- INICIO enable_cls_opt = True ---------------------")
        
        list_cls_ini = cls_ini_opt 
        
        list_cls_ini = list(dict.fromkeys(list_cls_ini)) # removiendo duplicados  

        list_cl_to_delete = []
        for COL in list_cls_ini:
     
            if COL not in X_train:   
                print(f" se eliminara la columna {COL} que no existe")                
                list_cl_to_delete.append(COL)
        
                
        list_cls_ini = hd.Diff(list_cls_ini, list_cl_to_delete) 
        
        list_cls = hd.Diff(X_train.columns.tolist(),list_cls_ini) 
        
        #print("list_cls: ", list_cls)
        list_sesgo = []
        last_best_value = 0
        
        for COLUMN in list_cls:            

            
            if max_iter_cls is not None:
            
                if len(list_sesgo)>=max_iter_cls:
                    break
            
            list_cls_ini.append(COLUMN)
            
            X_train_ = X_train.copy()     
            #print("X_train_ : ", X_train_.columns.tolist())
            #print("COLUMN : ", COLUMN)
            #print("list_cls_ini : ", list_cls_ini)
            
            X_train_ = X_train_[list_cls_ini]
            
            X_test_ = X_test.copy()             
            X_test_ = X_test_[list_cls_ini]
            
                    
            args["X_train"]=X_train_
            args["X_test"]=X_test_
            args["url"]=None
    
            model ,  y_prob_uno_train, y_prob_uno_test    = l.modelar(**args)
            
            kpis_train = generar_reporte(model,y_prob_uno_train,X_train,y_train,None,None,print_consola)
        
            kpis_test = generar_reporte(model,y_prob_uno_test,X_test,y_test,None,None,print_consola)
            
            
            f1_train = kpis_train[2]
            f1_test  = kpis_test[2] 
                
            SESGO = f1_train  - f1_test
            
            sesgo_var = {"COLUMN":COLUMN,"DIFF":SESGO,"F1_TRAIN":f1_train,"F1_TEST":f1_test }
            
            if SESGO > sesgo:
                list_cls_ini.remove(COLUMN)
                sesgo_var["SESGO"]=True
                sesgo_var["IMPROVEMENT"]=False
            else:
                sesgo_var["SESGO"]=False
                if f1_test > last_best_value:
                    last_best_value = f1_test
                    sesgo_var["IMPROVEMENT"]=True
                else:
                    list_cls_ini.remove(COLUMN)
                    sesgo_var["IMPROVEMENT"]=False
            print(sesgo_var)     
            list_sesgo.append(sesgo_var)
            
        
        g.save_json(list_sesgo,url+"/resumen_cls_sesgo.json")
        g.save_json(list_cls_ini,url+"/list_cls_sin_sesgo.json")           

                        
        args["X_train"]=X_train[list_cls_ini]
        args["X_test"]=X_test[list_cls_ini]
        args["url"]=url
        
        print("----------- FIN enable_cls_opt = True ---------------------")
        
        
    model ,  y_prob_uno_train, y_prob_uno_test    = l.modelar(**args)
    
    if early_stop is None:
        num_rounds_cv = num_rounds 
    else:
        num_rounds_cv = model.best_iteration()
        
  
    if cv : 
        
        if url is None:
            print ("SE NECESITA UNA URL PARA PROCESAR CROSS VALIDATION")
            
        else:
            print("INICIO CROSS VALIDATION: ")
            print(params)
            args_cv = { 'X_train':X_train, 'y_train': y_train, 'X_test': X_test, 'y_test': y_test, 'params':params,
                        "metric":metric,"num_rounds":num_rounds_cv,"random_state":random_state,
                        "api":api , "url":url , "cv_n_split" :cv_n_split, "shuffle_cv" :shuffle_cv,
                        "shuffle_cv_test_size" :shuffle_cv_test_size,
                        }      
            
            generate_cv_result(**args_cv)    

    
    kpis_train = generar_reporte(model,y_prob_uno_train,X_train,y_train,None,None,print_consola)
    
    kpis_test = generar_reporte(model,y_prob_uno_test,X_test,y_test,None,url,print_consola)
    
        

    umbral_opt=0
    kpis_opt = (0,0,0,0,0,0,0,0)
    
    if enable_umbral_opt:
        umbral_opt = g.get_opt_threshold(y_test,y_prob_uno_test)   
        kpis_opt = generar_reporte(model,y_prob_uno_test,X_test,y_test,umbral_opt,None,print_consola)
    
    print("Time elapsed: ", time.time() - start)
    
    return model ,  kpis_train, kpis_test , kpis_opt , umbral_opt , list_cls_ini



def generate_cv_result(X_train=None,y_train=None,X_test=None,y_test=None,api=None, num_rounds = None , metric=None, 
                       params =None ,  url=None, shuffle_cv = False ,shuffle_cv_test_size=None, cv_n_split=None,
                       random_state=42):
    
   print(X_train.shape)
   X_t_ = X_train.append(X_test)
   y_ = y_train.append(y_test)
   
   if shuffle_cv:
       folds = StratifiedShuffleSplit(n_splits=cv_n_split, test_size=shuffle_cv_test_size, random_state=random_state)
       print("StratifiedShuffleSplit")
   else:
   
       if cv_n_split is None:
           test_size = get_test_size(X_t_)
           n_s = round(1/test_size)
       else:
           n_s = cv_n_split      
         
       print("StratifiedKFold")
       folds = StratifiedKFold(n_splits=n_s, shuffle=True, random_state=random_state) 
   print("##########################################")
   print("params: : ", params)
   print("num_rounds_cv : ",num_rounds)
   print("##########################################")

   path_cv = os.path.join(url,"cv")  
   hg.validar_directorio(path_cv)  
   
   i = 0   
   for train_idx, valid_idx in folds.split(X_t_, y_):
       
       X_train_cv, y_train_cv  = X_t_.iloc[train_idx], y_.iloc[train_idx]
       X_valid_cv , y_valid_cv = X_t_.iloc[valid_idx], y_.iloc[valid_idx]
       
       args_cv = { 'X_train':X_train_cv, 'y_train': y_train_cv, 'X_test': X_valid_cv, 'y_test': y_valid_cv, 'params':params,
                   "metric":metric,"num_rounds":num_rounds,"early_stop":None,"log":0,"random_state":random_state,"api":api ,
                   "strategy":None, "url":None }  
       
       model_cv ,  y_prob_uno_train_cv, y_prob_uno_test_cv    = l.modelar(**args_cv)
       
       path_cv_test_filename = os.path.join(path_cv, f"y_test_cv_{i}.csv")      
       df_ = pd.DataFrame()
       df_["y_test_cv"] = y_valid_cv
       df_["y_prob_uno_test_cv"] = y_prob_uno_test_cv  
       df_.to_csv(path_cv_test_filename,index=False)  
       
   
       path_cv_train_filename = os.path.join(path_cv, f"y_train_cv_{i}.csv") 
       df_ = pd.DataFrame()
       df_["y_train_cv"] = y_train_cv
       df_["y_prob_uno_train_cv"] = y_prob_uno_train_cv       
       df_.to_csv(path_cv_train_filename,index=False)     
          
       i += 1 
       
             

def kpis_cross_validation(list_path=[], n_splits=5,url_out=None,postfix=""):
  
     

    list_train_filtracion = [] 
    list_train_subcobertura = [] 
    
    list_train_specificity = []    
    list_train_precision = []
    list_train_recall = []
    list_train_f1 = []
    list_train_average_precision = []
    list_train_roc_auc = []
    
    list_valid_filtracion = [] 
    list_valid_subcobertura = [] 
    
    list_valid_specificity = []
    list_valid_precision = []
    list_valid_recall = []
    list_valid_f1 = []
    list_valid_average_precision = []
    list_valid_roc_auc = []
    
    
    

    
    for i in range(n_splits):
        
       df_cv_fold = pd.DataFrame()           
       for path_ in list_path:
           print(" path_ :  ", path_)   
           df_cv_fold_ = pd.read_csv(f"{path_}/y_train_cv_{i}.csv", index_col=False)
           df_cv_fold = df_cv_fold.append(df_cv_fold_)
           
       y_train_cv = df_cv_fold.y_train_cv
       y_prob_uno_train_cv = df_cv_fold.y_prob_uno_train_cv       
       y_pred_train_cv = np.round(y_prob_uno_train_cv, 0)
       
       precision_train, recall_train, f1_train, support_train = precision_recall_fscore_support(y_train_cv, y_pred_train_cv ,average="binary",pos_label=1)
       train_average_precision = average_precision_score(y_train_cv, y_prob_uno_train_cv)      
       roc_auc_train = roc_auc_score(y_train_cv, y_prob_uno_train_cv) 
       
       filtracion_train = 1 - precision_train
       list_train_filtracion.append(filtracion_train)
       subcobertura_train = 1 - recall_train
       list_train_subcobertura.append(subcobertura_train)
       
       cm1 = confusion_matrix(y_train_cv,y_pred_train_cv)
       #total1=sum(sum(cm1))
       specificity_train = cm1[0,0]/(cm1[0,0]+cm1[0,1])       
    
       list_train_specificity.append(specificity_train)
        
       list_train_precision.append(precision_train)
       list_train_recall.append(recall_train)
       list_train_f1.append(f1_train)
       list_train_average_precision.append(train_average_precision)
       list_train_roc_auc.append(roc_auc_train)
       
       
       df_cv_fold = pd.DataFrame()           
       for path_ in list_path:
           print(" path_ :  ", path_)   
           df_cv_fold_ = pd.read_csv(f"{path_}/y_test_cv_{i}.csv", index_col=False)
           df_cv_fold = df_cv_fold.append(df_cv_fold_)      
        
       y_test_cv = df_cv_fold.y_test_cv
       y_prob_uno_test_cv = df_cv_fold.y_prob_uno_test_cv       
       y_pred_valid_cv = np.round(y_prob_uno_test_cv, 0)
       
       precision_test, recall_test, f1_test, support_test = precision_recall_fscore_support(y_test_cv, y_pred_valid_cv ,average="binary",pos_label=1)
       valid_average_precision = average_precision_score(y_test_cv, y_prob_uno_test_cv)      
       roc_auc_valid = roc_auc_score(y_test_cv, y_prob_uno_test_cv) 
       
              
       filtracion_valid = 1 - precision_test
       list_valid_filtracion.append(filtracion_valid)
       subcobertura_valid = 1 - recall_test
       list_valid_subcobertura.append(subcobertura_valid)
       
       cm11 = confusion_matrix(y_test_cv,y_pred_valid_cv)
       specificity_valid = cm11[0,0]/(cm11[0,0]+cm11[0,1])
       
       list_valid_specificity.append(specificity_valid)
               
       list_valid_precision.append(precision_test)
       list_valid_recall.append(recall_test)
       list_valid_f1.append(f1_test)
       list_valid_average_precision.append(valid_average_precision)
       list_valid_roc_auc.append(roc_auc_valid)
       
          
    mean_valid_subcobertura = statistics.mean(list_valid_subcobertura) 
    std_valid_subcobertura = statistics.pstdev(list_valid_subcobertura)
    min_valid_subcobertura = min(list_valid_subcobertura)           
    max_valid_subcobertura = max(list_valid_subcobertura)      
    mean_train_subcobertura = statistics.mean(list_train_subcobertura) 
    std_train_subcobertura = statistics.pstdev(list_train_subcobertura)
    min_train_subcobertura = min(list_train_subcobertura)           
    max_train_subcobertura = max(list_train_subcobertura)      
   
    mean_valid_filtracion = statistics.mean(list_valid_filtracion) 
    std_valid_filtracion = statistics.pstdev(list_valid_filtracion)
    min_valid_filtracion = min(list_valid_filtracion)           
    max_valid_filtracion = max(list_valid_filtracion)      
    mean_train_filtracion = statistics.mean(list_train_filtracion) 
    std_train_filtracion = statistics.pstdev(list_train_filtracion)
    min_train_filtracion = min(list_train_filtracion)           
    max_train_filtracion = max(list_train_filtracion)     

    mean_valid_specificity = statistics.mean(list_valid_specificity) 
    std_valid_specificity = statistics.pstdev(list_valid_specificity)
    min_valid_specificity = min(list_valid_specificity)           
    max_valid_specificity = max(list_valid_specificity)      
    mean_train_specificity = statistics.mean(list_train_specificity) 
    std_train_specificity = statistics.pstdev(list_train_specificity)
    min_train_specificity = min(list_train_specificity)           
    max_train_specificity = max(list_train_specificity)     

    mean_valid_precision = statistics.mean(list_valid_precision)           
    std_valid_precision = statistics.pstdev(list_valid_precision)
    min_valid_precision = min(list_valid_precision)           
    max_valid_precision = max(list_valid_precision)      
    mean_train_precision = statistics.mean(list_train_precision)           
    std_train_precision = statistics.pstdev(list_train_precision)
    min_train_precision = min(list_train_precision)           
    max_train_precision = max(list_train_precision)  
    
    
    mean_valid_recall = statistics.mean(list_valid_recall)           
    std_valid_recall = statistics.pstdev(list_valid_recall)
    min_valid_recall = min(list_valid_recall)           
    max_valid_recall = max(list_valid_recall)    
    mean_train_recall = statistics.mean(list_train_recall)           
    std_train_recall = statistics.pstdev(list_train_recall)
    min_train_recall = min(list_train_recall)           
    max_train_recall = max(list_train_recall) 
    
    
    mean_valid_f1 = statistics.mean(list_valid_f1)           
    std_valid_f1 = statistics.pstdev(list_valid_f1)
    min_valid_f1 = min(list_valid_f1)           
    max_valid_f1 = max(list_valid_f1)       
    mean_train_f1 = statistics.mean(list_train_f1)           
    std_train_f1 = statistics.pstdev(list_train_f1)   
    min_train_f1 = min(list_train_f1)           
    max_train_f1 = max(list_train_f1)   
    

    mean_valid_average_precision = statistics.mean(list_valid_average_precision) 
    std_valid_average_precision = statistics.pstdev(list_valid_average_precision)
    min_valid_average_precision = min(list_valid_average_precision)           
    max_valid_average_precision = max(list_valid_average_precision)     
    mean_train_average_precision = statistics.mean(list_train_average_precision) 
    std_train_average_precision = statistics.pstdev(list_train_average_precision) 
    min_train_average_precision = min(list_train_average_precision)           
    max_train_average_precision = max(list_train_average_precision)     
    
    
    mean_valid_roc_auc = statistics.mean(list_valid_roc_auc)           
    std_valid_roc_auc = statistics.pstdev(list_valid_roc_auc)
    min_valid_roc_auc = min(list_valid_roc_auc)           
    max_valid_roc_auc = max(list_valid_roc_auc)    
    mean_train_roc_auc = statistics.mean(list_train_roc_auc)           
    std_train_roc_auc = statistics.pstdev(list_train_roc_auc)
    min_train_roc_auc = min(list_train_roc_auc)           
    max_train_roc_auc = max(list_train_roc_auc)  

    
    result_kpis = [
                   {"Métrica":"Precisión","Splits":n_splits,"Datos":"Validación","Promedio":mean_valid_precision,"SD":std_valid_precision,"Min":min_valid_precision,"Max":max_valid_precision},
                   {"Métrica":"Precisión","Splits":n_splits,"Datos":"Entrenamiento","Promedio":mean_train_precision,"SD":std_train_precision,"Min":min_train_precision,"Max":max_train_precision},
                   
                   {"Métrica":"Sensibilidad","Splits":n_splits,"Datos":"Validación","Promedio":mean_valid_recall,"SD":std_valid_recall,"Min":min_valid_recall,"Max":max_valid_recall},
                   {"Métrica":"Sensibilidad","Splits":n_splits,"Datos":"Entrenamiento","Promedio":mean_train_recall,"SD":std_train_recall,"Min":min_train_recall,"Max":max_train_recall},
    
                   {"Métrica":"Especificidad","Splits":n_splits,"Datos":"Validación","Promedio":mean_valid_specificity,"SD":std_valid_specificity,"Min":min_valid_specificity,"Max":max_valid_specificity},               
                   {"Métrica":"Especificidad","Splits":n_splits,"Datos":"Entrenamiento","Promedio":mean_train_specificity,"SD":std_train_specificity,"Min":min_train_specificity,"Max":max_train_specificity},               
                                       
                   {"Métrica":"F1","Splits":n_splits,"Datos":"Validación","Promedio":mean_valid_f1,"SD":std_valid_f1,"Min":min_valid_f1,"Max":max_valid_f1},
                   {"Métrica":"F1","Splits":n_splits,"Datos":"Entrenamiento","Promedio":mean_train_f1,"SD":std_train_f1,"Min":min_train_f1,"Max":max_train_f1},
    
                   {"Métrica":"PR AUC","Splits":n_splits,"Datos":"Validación","Promedio":mean_valid_average_precision,"SD":std_valid_average_precision,"Min":min_valid_average_precision,"Max":max_valid_average_precision},
                   {"Métrica":"PR AUC","Splits":n_splits,"Datos":"Entrenamiento","Promedio":mean_train_average_precision,"SD":std_train_average_precision,"Min":min_train_average_precision,"Max":max_train_average_precision},
    
                   {"Métrica":"ROC AUC","Splits":n_splits,"Datos":"Validación","Promedio":mean_valid_roc_auc,"SD":std_valid_roc_auc,"Min":min_valid_roc_auc,"Max":max_valid_roc_auc},
                   {"Métrica":"ROC AUC","Splits":n_splits,"Datos":"Entrenamiento","Promedio":mean_train_roc_auc,"SD":std_train_roc_auc,"Min":min_train_roc_auc,"Max":max_train_roc_auc},               
                  
                   {"Métrica":"Filtración ","Splits":n_splits,"Datos":"Validación","Promedio":mean_valid_filtracion,"SD":std_valid_filtracion,"Min":min_valid_filtracion,"Max":max_valid_filtracion},
                   {"Métrica":"Filtración ","Splits":n_splits,"Datos":"Entrenamiento","Promedio":mean_train_filtracion,"SD":std_train_filtracion,"Min":min_train_filtracion,"Max":max_train_filtracion},               
                 
                   {"Métrica":"Subcobertura  ","Splits":n_splits,"Datos":"Validación","Promedio":mean_valid_subcobertura,"SD":std_valid_subcobertura,"Min":min_valid_subcobertura,"Max":max_valid_subcobertura},
                   {"Métrica":"Subcobertura ","Splits":n_splits,"Datos":"Entrenamiento","Promedio":mean_train_subcobertura,"SD":std_train_subcobertura,"Min":min_train_subcobertura,"Max":max_train_subcobertura},               
                  
                  ]
    
    df = pd.DataFrame(result_kpis)
    
    if url_out is None:
        return df
    else:
        df.to_excel(url_out+f"kpis_cross_validation_{postfix}.xlsx")  
 
 

def roc_curva_cross_validation(list_path=[], n_splits=5,url_out=None,postfix=""):
    
    tprs = []
    aucs = []
    mean_fpr = np.linspace(0, 1, 100)

    #list_path = ["CV","CV2"]
    
       
    for i in range(n_splits):   
        
       df_cv_fold = pd.DataFrame()   
        
       for path_ in list_path:
           #print(" path_ :  ", path_)
           df_cv_fold_ = pd.read_csv(f"{path_}/y_test_cv_{i}.csv", index_col=False)
           df_cv_fold = df_cv_fold.append(df_cv_fold_)
           
       y_test_cv = df_cv_fold.y_test_cv
       y_prob_uno_test_cv = df_cv_fold.y_prob_uno_test_cv
       
       fpr, tpr, thresholds = roc_curve(y_test_cv, y_prob_uno_test_cv)
       tprs.append(interp(mean_fpr, fpr, tpr))
       tprs[-1][0] = 0.0
       roc_auc = auc(fpr, tpr)
       aucs.append(roc_auc)
       #plt.plot(fpr, tpr, lw=1, alpha=0.3, label='ROC fold %d (AUC = %0.2f)' % (i, roc_auc))
       i += 1  
    
    #print("df_cv_fold.shape: ", df_cv_fold.shape)
    plt.plot([0, 1], [0, 1], linestyle='--', lw=1, color='r', label='Línea de base', alpha=.8)  
   
    mean_tpr = np.mean(tprs, axis=0)
    mean_tpr[-1] = 1.0
    mean_auc = auc(mean_fpr, mean_tpr)
    std_auc = np.std(aucs)
    plt.plot(mean_fpr, mean_tpr, color='b',label=r'Media ROC (AUC = %0.2f $\pm$ %0.2f)' % (mean_auc, std_auc),lw=1, alpha=.8)
   
    std_tpr = np.std(tprs, axis=0)
    tprs_upper = np.minimum(mean_tpr + std_tpr, 1)
    tprs_lower = np.maximum(mean_tpr - std_tpr, 0)
    plt.fill_between(mean_fpr, tprs_lower, tprs_upper, color='grey', alpha=.5, label=r' std. dev.')

    plt.xlim([-0.05, 1.05])
    plt.ylim([-0.05, 1.05])
    plt.xlabel('Tasa de falsos positivos')
    plt.ylabel('Tasa de verdaderos positivos')  
    plt.title(f"Curva ROC (n_splits:{n_splits})")
    plt.legend(loc="lower right")
    #plt.show()
    
    if url_out is None:
        plt.show()
    else:
        hg.validar_directorio(url_out)  
        plt.savefig(url_out+f'roc_curva_cv_{postfix}.png', bbox_inches='tight')
        plt.close()





def pr_curva_cross_validation(list_path=[], n_splits=5,url_out=None,postfix=""):

    
    y_real = []
    y_proba = []
       
    for i in range(n_splits):   
        
       df_cv_fold = pd.DataFrame()   
        
       for path_ in list_path:
           #print(" path_ :  ", path_)
           df_cv_fold_ = pd.read_csv(f"{path_}/y_test_cv_{i}.csv", index_col=False)
           df_cv_fold = df_cv_fold.append(df_cv_fold_)
                  
           
       y_test_cv = df_cv_fold.y_test_cv
       y_prob_uno_test_cv = df_cv_fold.y_prob_uno_test_cv       
       
       precision, recall, _ = precision_recall_curve(y_test_cv, y_prob_uno_test_cv)
             
       
       #plt.plot(recall, precision, lw=1, alpha=0.3,  label='PR fold %d (AUC = %0.2f)' % (i, average_precision_score(y_test_cv, y_prob_uno_test_cv)))
       plt.plot(recall, precision, lw=1, alpha=0.3)
       
       #plt.legend(loc="lower right")
       #plt.gca().get_legend().remove()
       
       y_real.append(y_test_cv)
       y_proba.append(y_prob_uno_test_cv)
       

       i += 1  
    
    y_real = np.concatenate(y_real)
    y_proba = np.concatenate(y_proba)    
    
    precision, recall, _ = precision_recall_curve(y_real, y_proba)
    plt.plot(recall, precision, color='b',label=r'Precisión-Recall (AUC = %0.2f)' % (average_precision_score(y_real, y_proba)), lw=2, alpha=.8)
    
    baseline = len(y_real[y_real==1]) / len(y_real)
    #baseline = 0.2
    #print(baseline)
    plt.plot([0, 1], [baseline, baseline], linestyle='--', label=r'Línea de base = %0.2f' %(baseline))
    
    #https://stats.stackexchange.com/questions/34611/meanscores-vs-scoreconcatenation-in-cross-validation

    plt.xlim([-0.05, 1.05])
    plt.ylim([-0.05, 1.05])
    plt.xlabel('Recall (Sensibilidad) ')
    plt.ylabel('Precisión')  
    plt.title(f"Curva Precisión-Recall (n_splits:{n_splits})")
    plt.legend(loc="upper right")
    #plt.gca().get_legend().remove()
    #plt.show()
    
    if url_out is None:
        plt.show()
    else:
        hg.validar_directorio(url_out)  
        plt.savefig(url_out+f'pr_curva_cv_{postfix}.png', bbox_inches='tight')
        plt.close()






def get_umbral_opt(list_path=[], n_splits=5,url_out=None,postfix="",intervalo = 0.001):

    
    y_real = []
    y_proba = []
       
    #opt_list = []
    
    for i in range(n_splits):   
        
       df_cv_fold = pd.DataFrame()   
        
       for path_ in list_path:
           #print(" path_ :  ", path_)
           df_cv_fold_ = pd.read_csv(f"{path_}/y_test_cv_{i}.csv", index_col=False)
           df_cv_fold = df_cv_fold.append(df_cv_fold_)
                  
           
       y_test_cv = df_cv_fold.y_test_cv
       y_prob_uno_test_cv = df_cv_fold.y_prob_uno_test_cv       
       
       
       y_real.append(y_test_cv)
       y_proba.append(y_prob_uno_test_cv)
       
       #umbral_opt = g.get_opt_threshold(y_test_cv,y_prob_uno_test_cv,intervalo=intervalo) 
       #print(umbral_opt)
       #opt_list.append(umbral_opt)
       

       i += 1  
    
    y_real = np.concatenate(y_real)
    y_proba = np.concatenate(y_proba)    
    
    umbral_opt = g.get_opt_threshold(y_real,y_proba,intervalo=intervalo)  
    
    #return umbral_opt , opt_list
    return umbral_opt




def distribucion_cross_validation(list_path=[], n_splits=5,url_out=None,postfix=""):

    
    y_real = []
    y_proba = []
       
    for i in range(n_splits):   
        
       df_cv_fold = pd.DataFrame()   
        
       for path_ in list_path:
           print(" path_ :  ", path_)
           df_cv_fold_ = pd.read_csv(f"{path_}/y_test_cv_{i}.csv", index_col=False)
           df_cv_fold = df_cv_fold.append(df_cv_fold_)
                  
           
       y_test_cv = df_cv_fold.y_test_cv
       y_prob_uno_test_cv = df_cv_fold.y_prob_uno_test_cv       
       

       y_real.append(y_test_cv)
       y_proba.append(y_prob_uno_test_cv)       

       i += 1  
       
    

    
    y_real = np.concatenate(y_real)
    y_proba = np.concatenate(y_proba)   
    
    
    sns.distplot(y_proba , hist = False, kde = True, 
                 color = 'darkblue', 
                 kde_kws={'linewidth': 3},
                 rug_kws={'color': 'black'})
    

    #plt.xlim([-0.05, 1.05])
    #plt.ylim([-0.05, 1.05])

    
    
    plt.title('Densidad de Kernel de la probabilidad estimada')
    plt.xlabel('Probabilidad Estimada')
    plt.ylabel('Densidad')
    
    #plt.gca().get_legend().remove()
    #plt.show()
    
    if url_out is None:
        plt.show()
    else:
        hg.validar_directorio(url_out)  
        plt.savefig(url_out+f'pr_curva_cv_{postfix}.png', bbox_inches='tight')
        plt.close()


def modelar_clasificacion_binaria_rscv(strategy, X_train,y_train=None,X_test=None,y_test=None,score_rs=None,params=None,param_dist=None, n_iter=None,n_jobs=None,url=None,print_consola=True):
    if url is not None:
        hg.validar_directorio(url)     

    if (strategy=="neg_bagging_fraction__lgb_model"):
        results, model , predicted_probas, params, best_params = nbf_lgb_model.modelar_rscv(X_train,y_train,X_test,y_test,score_rs,params,param_dist,n_iter,n_jobs,url)
        
    if (strategy=="scale_pos_weight__lgb_model"):
        results, model , predicted_probas, params, best_params = spw_lgb_model.modelar_rscv(X_train,y_train,X_test,y_test,score_rs,params,param_dist,n_iter,n_jobs,url)
       
    if (strategy=="custom_bagging__lgb_model"):
        results, model , predicted_probas, params, best_params = spw_lgb_model.modelar_rscv(X_train,y_train,X_test,y_test,score_rs,params,param_dist,n_iter,n_jobs,url)
       
    kpis = generar_reporte(model,predicted_probas,X_test,y_test,url,print_consola)
    
    return results, model , predicted_probas, params, best_params ,kpis
    

def generar_reporte(model,y_prob_uno, X_test, y_test,umbral_opt,url,print_consola): 
    
     
    kpis = hp.print_kpis_rendimiento_modelo(y_test,y_prob_uno,umbral_opt,url,print_consola)   
    '''
    if  isinstance(model, list)==False:
        if (print_consola):
            hp.print_shap_plot(model, X_test, url)   
    '''
    g.generate_summary_evaluation(X_test,y_prob_uno,y_test,url) 
    
    
    
    return kpis



    
    
def predecir_clasificacion_binaria(model, X=None, umbral=0.5):
    print("inicio predecir_clasificacion_binaria")
    
    if type(model)==lgb.basic.Booster:
        y_prob_uno = model.predict(X, num_iteration=model.best_iteration)      
        
    elif  isinstance(model, list)==False:    
        predicted_probas = model.predict_proba(X)
        y_prob_uno = predicted_probas[:,1]
    else:
        print("modelo es una lista")
        y_pred,y_prob_uno , predicted_probas = cb_lgb_model.predict_proba(model, X)
    
    y_pred_uno = np.where(y_prob_uno >= umbral, 1, 0).tolist()
    print("fin  predecir_clasificacion_binaria")
    return y_pred_uno, y_prob_uno



def get_result_df(KPIs_list,decimal=3):
    
    df = pd.DataFrame(columns=["Agrupación","Modelo","Total(E)","Total(E) Pos.","Total(V)","Total(V) Pos.",
                               'Precisión(E)', 'Sensibilidad(E)', 'Especificidad(E)','F1(E)','PR AUC(E)','ROC AUC(E)', 'Filtración(E)', 'Subcobertura(E)', 
                               'Precisión(V)', 'Sensibilidad(V)', 'Especificidad(V)','F1(V)','PR AUC(V)','ROC AUC(V)', 'Filtración(V)', 'Subcobertura(V)',
                               "umbral_opt",
                               'Precisión(Opt)','Sensibilidad(Opt)','Especificidad(Opt)','F1(Opt)','PR AUC(Opt)','ROC AUC(Opt)', 'Filtración(Opt)', 'Subcobertura(Opt)'])
    
    for idx, result  in enumerate(KPIs_list):
        mr = result[0]
        
        mod = result[1]
        
        t_train = result[2]
        
        t_train_class_1 = result[3]
        
        t_test = result[4]
        
        t_test_class_1 = result[5]
        
        Ks_train = result[6]
        
        Ks = result[7]
        umbral_opt = result[8]
        Ks_opt = result[9]
        
        result_rd = [mr,mod,t_train,t_train_class_1,t_test,t_test_class_1]+[round(num, decimal) for num in list(Ks_train)]+[round(num, decimal) for num in list(Ks)]+[umbral_opt]+[round(num_opt, decimal) for num_opt in list(Ks_opt)]
        df.loc[idx] = result_rd

        #df.loc[i] = list(KPIs)

    return df

def get_kpi_df_nacional(PATH_RESULT="resultado",grupos_grados=None,lista_mr=None):
    df_resumen = None
    #grupos_grados = {"1 prim": [4],"2 prim": [5], "3-5 prim": [6,7,8], "6 prim": [9], "1-4 sec": [10,11,12,13],"5 sec": [14]}
    #lista_mr = ["centro","norte","sur","oriente","lima"]
    
    list_df = []
    for key, grupo_grado  in grupos_grados.items():
        path = "{}/{}/{}".format(PATH_RESULT,key,"resultados.xlsx")
    
        df = pd.read_excel (path,index_col=0)
        idx = df.groupby(['Macro Región'])['Average Precision'].transform(max) == df['Average Precision']
        df = df[idx].copy()
        df["key_grupo_grado"] = key
        list_df.append(df)
    
    df_resumen = pd.concat(list_df)
    return df_resumen


def get_test_size(X_t=None,log=None):
    Total_X_t =  X_t.shape[0]
    Total_Test = 20000 # Cantidad minima de Test
    test_size = round(Total_Test/Total_X_t,5)
    
    min_test_size = 0.20
    if(test_size > min_test_size):
        test_size = 0.20 
    if  log is not None and log>0 :
        print("test_size : ", test_size)
    return test_size

def export_resultado_final_nacional(alto=0.75,medio=0.5,grupos_grados=None,lista_mr=None,path="resultado",
                                    delta_path=None,modelos=[]):
    if (grupos_grados is None):
        msg = "ERROR: No se ha especificado el parametro 'grupos_grados'"      
        raise Exception(msg)   
        
    if (lista_mr is None):
        msg = "ERROR: No se ha especificado el parametro 'lista_mr', que contiene la lista de macro regiones"      
        raise Exception(msg)   

    hg.validar_directorio(path)    
    if delta_path is not None:
        hg.validar_directorio(delta_path) 
        
    now = datetime.now()
    # dd/mm/YY H:M:S
    dt_string = now.strftime("%d%m%Y_%H%M%S")
    #"norte","sur"
    #grupos_grados = { "3-5 prim": [6,7,8]}
    #lista_mr = ["norte"]
    df_resumen = get_kpi_df_nacional(path+"/02.Modelo",grupos_grados)
        
    dt = {'COD_MOD':str,'COD_MOD_T':str,'ANEXO':int,'ANEXO_T':int,'EDAD':int,
      'N_DOC':str,'COD_MOD_T_MENOS_1':str,
      'ANEXO_T_MENOS_1':int,'NUMERO_DOCUMENTO_APOD':str,'ID_PERSONA':int}
    
    list_result = []
    
    if len(modelos)==0:
        modelos = ["neg_bagging_fraction__lgb_model","scale_pos_weight__lgb_model"] 
    
    columns = ['ID_PERSONA'] + modelos
    
    for key, grupo_grado  in grupos_grados.items():
        
        for macro_region in lista_mr:
            #print("obteniendo best model : ",macro_region," - ",key)
            best_model = df_resumen[(df_resumen.key_grupo_grado==key) & (df_resumen['Macro Región']==macro_region)]
            if best_model.shape[0]==0:
                msg = "ERROR: El archivo resultados.xlsx para el grupo de grados '"+key+"' no tiene resultados para la macro region: "+macro_region      
                raise Exception(msg)   
                
            best_model = best_model.Modelo.iloc[0]
            print(best_model," - ",macro_region," - ",key)
            if delta_path is None:
                specific_url = '{}/{}/{}/{}/{}'.format(path,"01.data_input",key,macro_region,"X_t_mas_1.csv")
            else:
                specific_url = '{}/{}/{}/{}/{}'.format(delta_path,"01.data_input",key,macro_region,"X_t_mas_1.csv")
                
            df=pd.read_csv(specific_url,dtype=dt, encoding="utf-8",usecols=columns) 
            df['RISK_SCORE'] = df[best_model] 
            list_result.append(df)
            
          
    df_nacional = pd.concat(list_result)
    df_nacional.drop_duplicates(subset ="ID_PERSONA",  keep = "first", inplace = True)
      
    df_nacional['PREDICCION']=None
    df_nacional.loc[(df_nacional['RISK_SCORE']>=alto) & (df_nacional['RISK_SCORE']<=1), 'PREDICCION'] = 3
    df_nacional.loc[(df_nacional['RISK_SCORE']>=medio) & (df_nacional['RISK_SCORE']<alto), 'PREDICCION'] = 2
    df_nacional.loc[df_nacional['RISK_SCORE']<medio, 'PREDICCION'] = 1
    df_nacional.PREDICCION = df_nacional.PREDICCION.astype(int)      
    
    cls_export = ["ID_PERSONA","RISK_SCORE",'PREDICCION']
    print("Total de registros : ",df_nacional.shape)
    if delta_path is None:
        url = "{}/{}/".format(path,"03.data_output")
        hg.validar_directorio(url)  
        filename = "nacional_{}.dta".format(dt_string)
        df_nacional[cls_export].to_stata(url+filename) 
    else:
        url = "{}/{}/".format(delta_path,"02.data_output")
        hg.validar_directorio(url) 
        filename = "nacional_{}.dta".format(dt_string)         
        df_nacional[cls_export].to_stata(url+filename)
    return df_nacional




def show_barplot_resultado_final_nacional(grupos_grados=None, lista_mr = None, scores = [] ,path = None,show=False):
    df_resumen = get_kpi_df_nacional(PATH_RESULT=path,grupos_grados=grupos_grados,lista_mr=lista_mr)
    
    list_result_total = []
    for macro_region in lista_mr:
        list_mr = []
        for score in scores:
            for key, grupo_grado  in grupos_grados.items():
                directory = path+"/"+str(key)
                hg.validar_directorio(directory) 
                final_path = directory+ "/resultados.xlsx"
                
                df = pd.read_excel(final_path, index_col=0)  
               
                best_model = df_resumen[(df_resumen.key_grupo_grado==key) & (df_resumen['Macro Región']==macro_region)]
                if best_model.shape[0]==0:
                    msg = "ERROR: El archivo resultados.xlsx para el grupo de grados '"+key+"' no tiene resultados para la macro region: "+macro_region      
                    raise Exception(msg)   
    
                valor = best_model[score].values[0]
                
                df_mr_grado = pd.DataFrame() 
                df_mr_grado['Valor'] = [valor] 
                df_mr_grado['Indicador'] = [score] 
                df_mr_grado['Grados'] = [key]        
                df_mr_grado['MR'] = [macro_region]
    
                list_mr.append(df_mr_grado)
        df_mr = pd.concat(list_mr)
        show_barplot_rf_n(df_mr,macro_region,path,show)
        list_result_total.append(df_mr)
    
    df_result_total = pd.concat(list_result_total)
    show_barplot_rf_n(df_result_total,"Nacional",path,show)
    return df_result_total

def show_barplot_rf_n(df,titulo_top_left="",dir_name=None,show=False):
    fig = plt.figure(figsize=(15, 6))
    ax = sns.barplot(x='Grados',y='Valor',data=df, hue='Indicador')
    
    for p in ax.patches:    
        height = p.get_height()
        #print(height)
        if math.isnan(height):
            height = 0
        #height = round(height,0)
        ax.text(p.get_x()+p.get_width()/2.,
                height ,
                '{:0.2f}'.format(height),
                ha="center") 
    plt.suptitle(titulo_top_left.upper()+' - Robustez por Grado', fontsize=20)
    ax.legend(ncol = 3, loc = 'best', bbox_to_anchor=(0.65, -0.1))
    
    
    if show :
        plt.show()
       
    filename =titulo_top_left+'_barplot.png'
    if len(dir_name.strip())==0 :
        full_dirname = filename
    else:
        if os.path.isdir(dir_name)==False:
            os.makedirs(dir_name)
        full_dirname = os.path.join(dir_name, filename)         
            
    plt.savefig(full_dirname, bbox_inches='tight')
    plt.close()

'''            
def split_x_y(ID_GRADO,macro_region,modalidad="EBR"):

    lista_regiones = get_macro_region(macro_region)
    list_join_n=[]
    list_join_n_mas_1=[]
    for region in lista_regiones:

        url_dir = "{}/{}/".format(region,ID_GRADO)
        print(url_dir)
        try:
            df_join_n , df_join_n_mas_1 = get_saved_join_data(url_dir,modalidad=modalidad)
        except:
            continue
        
        #df_join_n , df_join_n_mas_1 = get_saved_join_data(url_dir,modalidad=modalidad)
        df_join_n['REGION']= region
        df_join_n_mas_1['REGION']= region
        
        ############tempEEE#######
        df_join_n['D_REGION']= region
        df_join_n_mas_1['D_REGION']= region
        ########################
        
        
        print(region)
        print(df_join_n.DESERCION.value_counts())
        list_join_n.append(df_join_n)
        list_join_n_mas_1.append(df_join_n_mas_1)

    df_join_n = pd.concat(list_join_n)
    df_join_n_mas_1 = pd.concat(list_join_n_mas_1)

    fe_df(df_join_n,df_join_n_mas_1)

    X_train, X_test, y_train, y_test , X_t, X_t_eval, y_eval , ID_P_T,ID_P_T_MAS_1, y = tranform_data(df_join_n,df_join_n_mas_1,False)
    

    return X_train, X_test, y_train, y_test , X_t, X_t_eval, y_eval ,  ID_P_T,ID_P_T_MAS_1 , y
   

def get_saved_join_data(url_dir,sub_dir="data",modalidad="EBR"):
    
    if not url_dir:
        url_dir="../02.PreparacionDatos/03.Fusion/reporte_modelo/"+sub_dir+"/"
    else:
        url_dir = '{}/{}'.format("../02.PreparacionDatos/03.Fusion/reporte_modelo/"+sub_dir,url_dir)
        if not os.path.exists(url_dir):
            os.makedirs(url_dir)
        print("reporte generado en : "+url_dir)
    
    if (modalidad=="EBR"):
        specific_url = url_dir+"data.csv"
        specific_url_eval = url_dir+"data_eval.csv"
    else:
        specific_url = url_dir+"data_{}.csv".format(modalidad)
        specific_url_eval = url_dir+"data_eval_{}.csv".format(modalidad)        
    
    dt = {'COD_MOD':str,'COD_MOD_T':str,'ANEXO':int,'ANEXO_T':int,'EDAD':int,
          'N_DOC':str,'COD_MOD_T_MENOS_1':str,
          'ANEXO_T_MENOS_1':int,'NUMERO_DOCUMENTO_APOD':str,'ID_PERSONA':int}

    df=pd.read_csv(specific_url,dtype=dt, encoding="utf-8") 
    df_eval=pd.read_csv(specific_url_eval,dtype=dt, encoding="utf-8") 
    
    return df,df_eval

 ''' 